<?php
/**
* @package msta calendar Extension
*/

namespace msta\calendar\includes;

class CalendarBase {
  protected $db;
  protected $config;
  protected $user;
  protected $table_prefix;
  public $utc_timezone = null;
  public $user_timezone = null;
  static private $categories = null;
  static private $promoters = null;
  static private $promoterMembershipFetched = false;
  public $icsSpecialChars = array("\\", "\r", "\n", "\t", ",", ";");
  public $icsSpecialCharsEscaped = array("\\\\", "\\r", "\\n", "\\t", "\\,", "\\;");
  private $allowedKeys = null;

  public function __construct(\phpbb\db\driver\driver_interface $db, \phpbb\config\config $config, \phpbb\user $user) {
    $this->db = $db;
    $this->config = $config;
    $this->user = $user;
    $this->table_prefix = substr(USERS_TABLE, 0, -5);
    $this->utc_timezone = new \DateTimeZone('UTC');
    if ($this->user->data['user_timezone']) $this->user_timezone = new \DateTimeZone($this->user->data['user_timezone']);
    else $this->user_timezone = new \DateTimeZone($this->config['board_timezone']);
  }



  /*
   fetchCategories
   Loads the category info from database and stores it into the buffer.
  */
  private function fetchCategories () {
    $this->categories = array();
    $sql_result = $this->db->sql_query('SELECT * FROM '.$this->table_prefix.'calendar_category');
    foreach ($sql_result as $k => $v) { // just convert sql_result to true array
      $cloned_row = array();
      foreach ($v as $k2 => $v2) $cloned_row[$k2] = $v2;
      $this->categories['c'.$cloned_row['category_id']] = $cloned_row;
    }
    $sql_result->close();
  }

  /*
   categoryInfo
   Returns the database row of the event category with the given id. Buffers the categories for later use.
  */
  public function categoryInfo ($categoryId) {
    if ($this->categories == null) $this->fetchCategories();
    if (array_key_exists('c'.$categoryId, $this->categories)) return $this->categories['c'.$categoryId];
    else
      return array(
        'bcolor' => '000000',
        'big' => 0,
        'color' => '',
        'name' => '',
        'category_id' => $categoryId,
        'participants' => 0,
        'sort' => 0,
        'ics_additional' => '',
      );
  }

  /*
   categoryIdByName
   Returns the id of the category of the given name; 0 if not found. Buffers the categories for later use.
  */
  public function categoryIdByName ($categoryName) {
    if ($this->categories == null) $this->fetchCategories();
    foreach ($this->categories as $k => $v)
      if ($v['name'] == $categoryName) return intval($v['category_id']);
    return 0;
  }

  /*
   categoriesAsOptions
   Returns a string representing the options for a HTML-Select input for the categories.
  */
  public function categoriesAsOptions ($selectedId=0) {
    if ($this->categories == null) $this->fetchCategories();
    $result = '';
    foreach ($this->categories as $k => $v)
      $result .= '<OPTION value="'.intval($v['category_id']).'"'.(($selectedId == $v['category_id'])?' selected':'').' participants="'.$v['participants'].'">'.$v['name'].'</OPTION>';
    return $result;
  }



  private function fetchPromoters () {
    $this->promoters = array();
    $sql_result = $this->db->sql_query('SELECT group_id, group_name, group_type, calendar_settings, calendar_category FROM '.$this->table_prefix.'groups WHERE calendar_settings <> 0');
    foreach ($sql_result as $k => $v) { // just convert sql_result to true array
      if (($v['calendar_settings'] & 1) == 0) continue; // for this group, publishing of events is not allowed. Don't even list it!
      $cloned_row = array();
      foreach ($v as $k2 => $v2) $cloned_row[$k2] = $v2;
      $cloned_row['publishAllow'] = false; // indicates, that the current user is allowed to publish events for this group
      $cloned_row['userIsMember'] = false; // indicates, that the current user is member of this group
      $this->promoters['p'.$cloned_row['group_id']] = $cloned_row;
    }
    $sql_result->close();
  }

  private function fetchPromoterMemberships () {
    $memberships = $this->db->sql_query('SELECT * FROM '.$this->table_prefix.'user_group WHERE user_id = '.$this->user->data['user_id'].' AND user_pending = 0');
    while ($row = $this->db->sql_fetchrow($memberships)) { // determine the groups, in which the user is member and in which the user can create public events
      $gk = 'p'.$row['group_id'];
      if (!array_key_exists($gk, $this->promoters)) continue;
      $this->promoters[$gk]['userIsMember'] = true;
      if (($this->promoters[$gk]['calendar_settings'] & 2) == 2 || (($this->promoters[$gk]['calendar_settings'] & 4) == 4 && $row['group_leader'] == 1)) $this->promoters[$gk]['publishAllow'] = true;
    }
    $this->promoterMembershipFetched = true;
  }

  public function getPromoterIDs ($membershipsOnly = false) {
    if (!$this->promoters) $this->fetchPromoters();
    if (!$this->promoterMembershipFetched && $membershipsOnly) $this->fetchPromoterMemberships();
    $ids = array();
    foreach ($this->promoters as $k => $v)
      if (!$membershipsOnly || $v['userIsMember'])
        $ids[] = intval($v['group_id']);
    return $ids;
  }

  public function promoterIsValid ($groupId, $membershipsOnly = false) {
    if (!$this->promoters) $this->fetchPromoters();
    if (!$this->promoterMembershipFetched && $membershipsOnly) $this->fetchPromoterMemberships();
    return array_key_exists('p'.$groupId, $this->promoters) && ($this->promoters['p'.$groupId]['calendar_settings'] & 1) == 1 && (!$membershipsOnly || $this->promoters['p'.$groupId]['userIsMember']);
  }

  /*
   returns true, if then given promoter group is valid and the current user is allowed to publish events for this promoter group.
  */
  public function promoterCanPublish ($groupId, $adminOrGlobalMod = false) {
    if (!$this->promoters) $this->fetchPromoters();
    if (!$this->promoterMembershipFetched && !$adminOrGlobalMod) $this->fetchPromoterMemberships();
    return
      array_key_exists('p'.$groupId, $this->promoters) &&  // this ensures, that the given group exists
      (($this->promoters['p'.$groupId]['calendar_settings'] & 1) == 1) &&  // this ensures, that the given group is allowed to promote events
      ($adminOrGlobalMod || $this->promoters['p'.$groupId]['publishAllow']);
  }

  public function promoterInfo ($groupId) {
    if (!$this->promoters) $this->fetchPromoters();
    if (array_key_exists('p'.$groupId, $this->promoters)) return $this->promoters['p'.$groupId];
    else
      return array(
        'group_id' => 0,
        'group_name' => '',
        'group_type' => 0,
        'calendar_settings' => 0,
        'calendar_category' => '',
      );
  }

  public function promoterIdByName ($promoterName) {
    if (!$this->promoters) $this->fetchPromoters();
    foreach ($this->promoters as $k => $v)
      if ($v['group_name'] == $promoterName) return intval($v['group_id']);
    return 0;
  }

  public function promotersAsOptions ($membershipsOnly = false, $publishAllowInfo = false, $selectedGroupId = 0) {
    if (!$this->promoters) $this->fetchPromoters();
    if (!$this->promoterMembershipFetched && ($membershipsOnly || $publishAllowInfo)) $this->fetchPromoterMemberships();
    $opt = '<OPTION value="0" publish="no"'.((0 == $selectedGroupId)?' selected':'').'> - </OPTION>';
    foreach ($this->promoters as $k => $v)
      if (!$membershipsOnly || $v['userIsMember'])
        $opt .= '<OPTION value="'.$v['group_id'].'"'.(($publishAllowInfo)?' publish="'.(($v['publishAllow'])?'yes':'no').'"':'').(($v['group_id'] == $selectedGroupId)?' selected':'').'>'.htmlspecialchars($v['group_name']).'</OPTION>';
    return $opt;
  }



  /*
   ensureInt
   Ensure that obj[key] is an integer between min and max
  */
  private function ensureInt (&$obj, $key, $min, $max, $prefereMax = false) {
    $this->allowedKeys[] = $key;
    if (array_key_exists($key, $obj)) {
      $obj[$key] = intval($obj[$key]);
      if ($obj[$key] > $max) $obj[$key] = $max;
      if ($obj[$key] < $min)
        if ($prefereMax) $obj[$key] = $max;
        else $obj[$key] = $min;
    } else if ($prefereMax) $obj[$key] = $max;
    else $obj[$key] = $min;
  }

  /*
   ensureString
   Ensure that obj[key] is a string with the given maximum length
  */
  private function ensureString (&$obj, $key, $maxLength) {
    $this->allowedKeys[] = $key;
    if (array_key_exists($key, $obj)) {
      $obj[$key] = strval($obj[$key]);
      if (strlen($obj[$key]) > $maxLength) $obj[$key] = substr($obj[$key], 0, $maxLength);
    } else $obj[$key] = '';
  }

  /*
   ensureDate
   Ensure that obj[key] is either an empty string or of the form yyyy-mm-dd
  */
  private function ensureDate (&$obj, $key) {
    $this->allowedKeys[] = $key;
    if (array_key_exists($key, $obj)) {
      $spl = explode('-', strval($obj[$key]));
      for ($i = 0; $i < 3; $i++)
        if ($i >= count($spl)) $spl[$i] = 0;
        else $spl[$i] = intval($spl[$i]);
      if ($spl[1] <= 0 || $spl[2] <= 0) {
        $obj[$key] = ''; // invalid date value
        return;
      }
      if ($spl[1] > 12) $spl[1] = 12;
      if ($spl[2] > 31 && ($spl[1] == 1 || $spl[1] == 3 || $spl[1] == 5 || $spl[1] == 7 || $spl[1] == 8 || $spl[1] == 10 || $spl[1] == 12)) $spl[2] = 31;
      else if ($spl[2] > 30 && ($spl[1] == 4 || $spl[1] == 6 || $spl[1] == 9 || $spl[1] == 11)) $spl[2] = 30;
      else if ($spl[2] > 28 && $spl[1] == 2) {
        $leap = ($spl[0] % 4 == 0);
        if ($leap && ($spl[0] % 100 == 0)) $leap = false;
        if (!$leap && ($spl[0] % 400 == 0)) $leap = true;
        if ($leap && $spl[2] > 29) $spl[2] = 29;
        if (!$leap && $spl[2] > 28) $spl[2] = 28;
      }
      $spl[0] = strval($spl[0]);
      $spl[1] = strval($spl[1]);
      $spl[2] = strval($spl[2]);
      while (strlen($spl[0]) < 4) $spl[0] = '0'.$spl[0];
      while (strlen($spl[1]) < 2) $spl[1] = '0'.$spl[1];
      while (strlen($spl[2]) < 2) $spl[2] = '0'.$spl[2];
      $obj[$key] = $spl[0].'-'.$spl[1].'-'.$spl[2];
    } else $obj[$key] = ''; // invalid date value
  }

  /*
   ensureTime
   Ensure that obj[key] is either an empty string or of the form hh:mm
  */
  private function ensureTime (&$obj, $key) {
    $this->allowedKeys[] = $key;
    if (array_key_exists($key, $obj)) {
      if (strval($obj[$key]) == '' || strval($obj[$key]) == '--:--') {
       $obj[$key] = ''; // invalid time value
       return;
      }
      $spl = explode(':', strval($obj[$key]));
      $spl[0] = abs(intval($spl[0]));
      if (count($spl) >= 2) $spl[1] = abs(intval($spl[1]));
      else $spl[1] = 0;
      if ($spl[0] > 23) $spl[0] = 23;
      if ($spl[1] > 59) $spl[1] = 59;
      $spl[0] = strval($spl[0]);
      $spl[1] = strval($spl[1]);
      while (strlen($spl[0]) < 2) $spl[0] = '0'.$spl[0];
      while (strlen($spl[1]) < 2) $spl[1] = '0'.$spl[1];
      $obj[$key] = $spl[0].':'.$spl[1];
    } else $obj[$key] = ''; // invalid time value
  }

  /*
   checkEvent
   Corrects wrong values of the event; can also be used to initialize a new event from an empty array
  */
  public function checkEvent (&$date) {
    $this->allowedKeys = array();
    $this->ensureInt($date,    'post_id', 0, 16777215);
    $this->ensureString($date, 'event_name', 255);
    $this->ensureString($date, 'location', 255);
    $this->ensureString($date, 'url', 255);
    $this->ensureString($date, 'description', 2000);
    $this->ensureInt($date,    'category', 0, 65535);
    $this->ensureInt($date,    'promoter', 0, 16777215);
    $this->ensureInt($date,    'public', 0, 1);
    $this->ensureDate($date,   'date_from');
    $this->ensureTime($date,   'time_from');
    $this->ensureDate($date,   'date_to');
    $this->ensureTime($date,   'time_to');
    $this->ensureInt($date,    'repeat_mode', 0, 9);
    $this->ensureInt($date,    'repeat_nr', -2147483648, 2147483647);
    $this->ensureInt($date,    'canceled', 0, 1);
    $this->ensureInt($date,    'participants', 0, 3);
    $this->ensureInt($date,    'anniversary', 0, 1);
    $this->ensureString($date, 'uid', 255);
    foreach ($date as $k => $v)
      if (!in_array($k, $this->allowedKeys))
        $date[$k] = null; // this will not be in sql-query
    // ensure to > from
    if ($date['date_from'] == '') {
     $date['date_to'] = '';
     $date['time_to'] = '';
     $date['time_from'] = '';
    } else if ($date['date_to'] == '') {
      if ($date['repeat_mode'] == 0) $date['date_to'] = $date['date_from'];
    } else {
      $from = explode('-', $date['date_from']);
      $to = explode('-', $date['date_to']);
      for ($i = 0; $i < 3; $i++)
        if (intval($to[$i]) < intval($from[$i])) $date['date_to'] = $date['date_from'];
        else if (intval($to[$i]) > intval($from[$i])) break;
    }
    if ($date['time_from'] == '' || $date['time_to'] == '') {
     $date['time_from'] = '';
     $date['time_to'] = '';
    } else if ($date['date_from'] == $date['date_to']) {
      $from = explode(':', $date['time_from']);
      $to = explode(':', $date['time_to']);
      for ($i = 0; $i < 2; $i++)
        if (intval($to[$i]) < intval($from[$i])) {
          $date['time_from'] = '';
          $date['time_to'] = '';
        } else if (intval($to[$i]) > intval($from[$i])) break;
    }
    // ensure URL is correct
    $date['url'] = filter_var($date['url'], FILTER_SANITIZE_URL);
    if (!filter_var($date['url'], FILTER_VALIDATE_URL)) $date['url'] = '';
  }
}
